from setuptools import setup, find_packages

setup(
    name="fkir",
    version="0.0.6",
    author="oPluss",
    author_email="opluss@qq.com",
    description="The fucking ir utils",
    url="https://github.com/OYCN/fuck-ir",
    packages=find_packages(),
    install_requires=["networkx"],
)
